@@ -3,7 +3,7 @@ Created on May 26 01:47:13 2022
 """
 
 import tkinter as tk
-from typing import Any, Union
+from typing import Any, Optional, Union
 
 import numpy as np
 from astropy.constants import codata2018 as c_2018
@@ -11,10 +11,10 @@ from astropy.units.quantity import Quantity
 
 try:
     from . import tk_functions as tk_f
-    from . import celestial_objects
 except ImportError:
     import tk_functions as tk_f
-    import celestial_objects
+
+N_float = Union[float, np.float_, np.ndarray]
 
 
 def convert(parameter: Quantity, change_to: str) -> Quantity:
@@ -34,23 +34,28 @@ def convert(parameter: Quantity, change_to: str) -> Quantity:
         The celestial object parameter with changed unit.
 
     """
-    return parameter.to(change_to)
+    return parameter.to(change_to) if not change_to == '' else parameter
 
 
-class GetOrbitalParameters:
+def get_absolute_magnitude(apparent_magnitude: float, distance: Quantity) -> float:
+    """
+    Calculates the absolute magnitude for the celestial object.
 
-    def __init__(self, a_0: float, ecc: float):
-        self.semi_major_axis = a_0
-        self.eccentricity = ecc
+    Parameters
+    -------
+    apparent_magnitude: float
+        Apparent magnitude of the celestial object.
+    distance: Quantity
+        Distance of the celestial object from the Earth.
 
-    def get_apo_distance(self):
-        return self.semi_major_axis * (1 - self.eccentricity)
+    Returns
+    -------
+    float:
+        Absolute magnitude of the celestial object.
 
-    def get_peri_distance(self):
-        return self.semi_major_axis * (1 + self.eccentricity)
-
-    def get(self):
-        return self.get_apo_distance(), self.get_peri_distance()
+    """
+    distance = distance.to('pc')
+    return apparent_magnitude - 5 * np.log10(distance.value) + 5
 
 
 class GetPhysicalParameters:
@@ -59,15 +64,15 @@ class GetPhysicalParameters:
 
     """
 
-    def __init__(self, mass: float, radius: float):
+    def __init__(self, mass: Quantity, radius: Quantity):
         """
-        Initialization function for PhysicalParameter class.
+        Initialization function for GetPhysicalParameter class.
 
         Parameters
         ----------
-        mass : TYPE
+        mass : Quantity
             Mass of the celestial object.
-        radius : TYPE
+        radius : Quantity
             Radius of the celestial object.
 
         Returns
@@ -148,8 +153,8 @@ class GetPhysicalParameters:
                 self.surface_gravity(), self.escape_velocity())
 
 
-def comparison(c_win: Union[tk.Tk, tk.Toplevel, tk.Frame], p_ojb: Any, c_obj: Any,
-               c_lbl: str, c_type: str, column: float, reset: bool = False):
+def comparison(c_win: Union[tk.Tk, tk.Toplevel, tk.Frame], primary_obj: Any, sec_obj: Any,
+               sec_lbl: str, comparison_type: str, column: int, reset: bool = False):
     """
     Compares the attributes of given celestial object (o_obj) with comparison celestial
     object (c_obj).
@@ -158,15 +163,15 @@ def comparison(c_win: Union[tk.Tk, tk.Toplevel, tk.Frame], p_ojb: Any, c_obj: An
     ----------
     c_win : Union[tk.Tk, tk.Toplevel, tk.Frame]
         tk.Tk, tk.Toplevel window or tk.Frame to build the object inside.
-    p_ojb : Any
+    primary_obj : Any
         The object clas to which the comparison is being done.
-    c_obj : Any
+    sec_obj : Any
         The object class with which the comparison is being done.
-    c_lbl : str
+    sec_lbl : str
         Text representing the comparison celestial object.
-    c_type: str
-        Whether the comparison should be of physical or orbital parameters.
-    column: float
+    comparison_type: str
+        Whether the comparison should be of physical, orbital or observational parameters.
+    column: int
         Specify the column number where the equivalencies should be placed.
     reset : bool, optional
         Option to set the comparison entries to null. The default is False.
@@ -177,35 +182,122 @@ def comparison(c_win: Union[tk.Tk, tk.Toplevel, tk.Frame], p_ojb: Any, c_obj: An
 
     """
     # get all the class attributes
-    attributes = p_ojb.__dict__.keys()
+    attributes = primary_obj.__dict__.keys()
     # get their number
     num_attributes = len(attributes)
 
-    if c_type == 'physical':
-        c_obj = c_obj.PhysicalParameters()
+    if comparison_type == 'physical':
+        sec_obj = sec_obj.PhysicalParameters()
+    elif comparison_type == 'orbital':
+        sec_obj = sec_obj.OrbitalParameters()
     else:
-        c_obj = c_obj.OrbitalParameters()
+        sec_obj = sec_obj.ObservationalParameters()
 
     # divide the celestial object attribute values to that of comparison celestial
     # object attributes
 
-    try:
-        out = [p_ojb.__getattribute__(attr) / c_obj.__getattribute__(attr) for attr
-               in attributes]
-    except (ZeroDivisionError, RuntimeWarning):
-        if type(c_obj) == celestial_objects.Sun.OrbitalParameters:
-            out = [1] * num_attributes
+    out = []
+    for attr in attributes:
+        if primary_obj.__getattribute__(attr) is None:
+            ratio = None
+        elif attr in ['apparent_magnitude', 'absolute_magnitude']:
+            ratio = sec_obj.__getattribute__(attr)
+            ratio -= primary_obj.__getattribute__(attr)
+
+            ratio = 100**(ratio / 5)
+        else:
+            ratio = primary_obj.__getattribute__(attr) / sec_obj.__getattribute__(attr)
+
+        out.append(ratio)
 
     # place the entries on the comparison window or reset them
     for value, num in zip(out, range(1, num_attributes + 1)):
-        if 0 < value <= 0.001:
-            value = f'{abs(value):.5e} × {c_lbl}'
+        if value is None:
+            value = None
+        elif 0 < value <= 0.001:
+            value = f'{abs(value):.5e} × {sec_lbl}'
+        elif value > int(1e9):
+            value = f'{np.round(abs(value), 9):.5E} × {sec_lbl}'
         else:
-            value = f'{np.round(abs(value), 5)} × {c_lbl}'
+            value = f'{np.round(abs(value), 9)} × {sec_lbl}'
 
-        if not reset:
-            tk_f.entry_placement(window=c_win, value=value, row=num, columns=column,
-                                 width=25)
-        else:
-            tk_f.entry_placement(window=c_win, value='', row=num, columns=column,
-                                 width=25)
+        value = value if not reset else ''
+
+        tk_f.entry_placement(window=c_win, value=value, row=num, columns=column, width=25)
+
+
+def Q(value: Union[float, np.ndarray], unit: str) -> Quantity:
+    """
+    This function initializes the value, unit pair of physical quantities.
+
+    Parameters
+    ----------
+    value : float
+        The value of the physical quantity.
+    unit : str
+        The unit of the physical quantity.
+
+    Returns
+    -------
+    Quantity
+        Physical quantity with value and unit.
+
+    """
+    return Quantity(value=value, unit=unit)
+
+
+def ifNone(val: float = None, unit: str = None) -> Optional[Quantity]:
+    """
+    Gives the output for input value and its units, or returns None
+
+    Parameters
+    ----------
+    val : float, optional
+        Magnitude of the physical quantity.
+    unit : str, optional
+        Unit of the physical quantity.
+
+    Returns
+    -------
+    Quantity or None
+
+    """
+    return None if None in [val, unit] else Q(value=val, unit=unit)
+
+
+pars_ = {'t_orb': [('s', 'hr', 'day', 'yr', 'Myr'), 'day'],
+         'v_orb': [('cm/s', 'm/s', 'km/s', 'km/h'), 'km/s'],
+         'm_anom': [('deg', 'rad'), 'deg'],
+         'incl': [('deg', 'rad'), 'deg'],
+         'long': [('deg', 'rad'), 'deg'],
+         'arg': [('deg', 'rad'), 'deg'],
+         'tilt': [('deg', 'rad'), 'deg'],
+         'dist': [('cm', 'm', 'km', 'Gm', 'AU', 'lyr', 'pc'), 'km'],
+         'size': [('arcsec', 'arcmin', 'deg', 'rad'), 'arcsec']
+         }
+
+
+def get_options(value: Any, prop: str) -> tuple:
+    """
+    Chooses options and default values for the given physical quantity.
+
+    Parameters
+    ----------
+    value : Any
+        Physical quantity.
+    prop : str
+        String containing the property shortcode to check against.
+
+    Returns
+    -------
+    tuple
+        Tuple containing the options and the default value is returned
+
+    """
+    if value is None:
+        option_ = tuple()
+        default_ = ''
+    else:
+        option_, default_ = pars_[prop]
+
+    return option_, default_
